#
# raccp.rb
#
#   Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Library General Public License.
#

require 'racc/parser'
require 'amstd/symbol'


module Racc

  class RaccParser < Parser

    def initialize( racc )
      @racc = racc
      @interf    = racc.interf
      @ruletable = racc.ruletable
    end


    attr :start
    attr :classname

    attr :filename

    def parse( str, fname, mydebug = false )
      fname.must String
      
      @scanner = RaccScanner.new( str )
      @scanner.debug    = @yydebug  = mydebug
      @scanner.filename = @filename = fname

      do_parse
    end


    private


    def next_token
      @scanner.scan
    end

    def on_error( tok, val, _values )
      case val
      when Symbol then v = val.id2name
      when String then v = val
      else
        v = val.inspect
      end
      raise ParseError,
%=parse error:#{@filename}:#{@scanner.lineno}: unexpected token '#{v}'=
    end

##### racc 0.11.0 generates ###

Racc_reduce_table = [
 0, 0, :racc_error,
 6, 20, :_reduce_1,
 0, 21, :_reduce_none,
 2, 21, :_reduce_none,
 3, 23, :_reduce_4,
 1, 23, :_reduce_none,
 2, 23, :_reduce_6,
 2, 24, :_reduce_7,
 3, 24, :_reduce_8,
 3, 25, :_reduce_9,
 3, 25, :_reduce_10,
 1, 27, :_reduce_none,
 2, 27, :_reduce_none,
 2, 28, :_reduce_13,
 2, 28, :_reduce_14,
 2, 28, :_reduce_15,
 1, 29, :_reduce_16,
 2, 29, :_reduce_17,
 1, 26, :_reduce_18,
 1, 26, :_reduce_19,
 2, 22, :_reduce_none,
 3, 22, :_reduce_none,
 3, 30, :_reduce_22,
 3, 30, :_reduce_23,
 0, 31, :_reduce_24,
 2, 31, :_reduce_25,
 1, 32, :_reduce_26,
 1, 32, :_reduce_27,
 1, 32, :_reduce_28,
 2, 32, :_reduce_29 ]

Racc_reduce_n = 30

Racc_shift_n = 53

Racc_action_table = [
     2,     2,    -1,   -30,     0,     3,    -1,   -30,     3,     4,
    -1,   -30,     0,     5,    -1,   -30,    -1,    -2,    -1,    53,
     4,    13,     6,    12,     7,    11,     9,    10,    10,     9,
    -1,   -30,    -1,    -5,    -1,    -3,    11,    18,    12,    17,
    13,    16,    -1,   -30,    11,    18,    12,    17,    13,    16,
    -1,   -30,     3,    20,    -1,   -30,     3,    24,     8,    23,
    -1,   -30,     3,    27,    -1,   -30,    -1,   -11,     9,    29,
    11,    18,    12,    17,    13,    16,    -1,   -30,     3,    24,
     8,    23,    -1,   -30,     3,    24,     8,    23,    -1,   -30,
     3,    24,     8,    23,    -1,   -30,    10,    34,    11,    18,
    12,    17,    13,    16,    -1,   -30,    -1,    -6,     8,    35,
    -1,   -30,     5,    37,     3,    24,     8,    23,    -1,   -30,
    -1,   -19,    -1,   -18,    14,    38,    16,    39,    -1,   -30,
     5,    41,     3,    27,    -1,   -30,    15,    42,    -1,   -30,
    -1,   -12,    -1,   -10,    -1,   -16,     3,    24,     8,    23,
    -1,   -15,     3,    24,     8,    23,    -1,   -14,     3,    24,
     8,    23,    -1,   -13,    -1,    -9,    -1,    -7,     8,    44,
    -1,   -30,    -1,    -4,    -1,   -20,    -1,   -24,    14,    46,
    16,    39,    -1,   -30,    -1,    -1,    -1,   -24,    -1,   -17,
    -1,    -8,     3,    51,     8,    50,    17,    49,    18,    52,
    -1,   -23,    -1,   -21,     3,    51,     8,    50,    17,    49,
    18,    52,    -1,   -22,    -1,   -25,    -1,   -28,    -1,   -27,
    -1,   -26,     3,    24,     8,    23,    -1,   -30,    -1,   -29 ]

Racc_action_table_ptr = [
     0,     4,     8,    12,    16,    18,    20,    32,    34,    36,
    44,    52,    56,    62,    66,    68,    78,    84,    90,    96,
   106,   108,   112,   120,   122,   124,   130,   136,   140,   142,
   144,   146,   152,   158,   164,   166,   168,   172,   174,   176,
   178,   184,   186,   188,   190,   192,   202,   204,   214,   216,
   218,   220,   222,   228 ]


Racc_goto_table = [
    20,     1,    21,     6,    25,     7,    23,     8,    28,    14,
    27,    15,    28,    14,    27,    19,    26,    21,    24,    22,
    30,    25,    22,    26,    28,    28,    26,    30,    29,    31,
    26,    30,    29,    32,    26,    30,    29,    33,    28,    28,
    26,    36,    30,    40,    26,    43,    26,    43,    26,    43,
    31,    45,    31,    47,    32,    48,    32,    48,    26,    53,
    -1,    -1 ]

Racc_goto_table_ptr = [
     0,    -1,    -1,    -1,     2,    -1,     4,    -1,    -1,     8,
    12,    -1,    16,    20,    -1,    24,    26,    30,    34,    38,
    -1,    -1,    40,    -1,    -1,    -1,    42,    -1,    -1,    -1,
    -1,    44,    46,    48,    -1,    -1,    -1,    -1,    -1,    50,
    -1,    -1,    52,    -1,    -1,    54,    -1,    56,    -1,    -1,
    -1,    -1,    58,    -1 ]


Racc_token_table = {
 false => 0,
 Object.new => 1,
 :CLASS => 2,
 :TOKEN => 3,
 :RULE => 4,
 :XEND => 5,
 :XTOKEN => 6,
 :START => 7,
 :STRING => 8,
 :PRECHIGH => 9,
 :PRECLOW => 10,
 :LEFT => 11,
 :RIGHT => 12,
 :NONASSOC => 13,
 ";" => 14,
 ":" => 15,
 "|" => 16,
 :ACTION => 17,
 "=" => 18 }

Racc_debug_parser = false

##### racc system variables end #####

 # reduce 0 omitted

 module_eval( '', '(build)', 82 ) {
  def _reduce_1( val, _values, result )
      @interf.end_register_rule
      @classname = (Integer === val[1]) ? val[1].id2name : val[1]
   result
  end
 }

 # reduce 2 omitted

 # reduce 3 omitted

 module_eval( '', '(build)', 93 ) {
  def _reduce_4( val, _values, result )
      @interf.end_register_conv
   result
  end
 }

 # reduce 5 omitted

 module_eval( '', '(build)', 99 ) {
  def _reduce_6( val, _values, result )
      if @start then
        raise ParseError, "start rule defined twice"
      end
      @ruletable.start = @start = @interf.get_token( val[1] )
   result
  end
 }

 module_eval( '', '(build)', 107 ) {
  def _reduce_7( val, _values, result )
      @interf.register_conv( val[0], val[1] )
   result
  end
 }

 module_eval( '', '(build)', 111 ) {
  def _reduce_8( val, _values, result )
      @interf.register_conv( val[1], val[2] )
   result
  end
 }

 module_eval( '', '(build)', 116 ) {
  def _reduce_9( val, _values, result )
      @interf.end_register_prec( true )
   result
  end
 }

 module_eval( '', '(build)', 120 ) {
  def _reduce_10( val, _values, result )
      @interf.end_register_prec( false )
   result
  end
 }

 # reduce 11 omitted

 # reduce 12 omitted

 module_eval( '', '(build)', 130 ) {
  def _reduce_13( val, _values, result )
      @interf.register_prec( :Left, val[1] )
   result
  end
 }

 module_eval( '', '(build)', 134 ) {
  def _reduce_14( val, _values, result )
      @interf.register_prec( :Right, val[1] )
   result
  end
 }

 module_eval( '', '(build)', 138 ) {
  def _reduce_15( val, _values, result )
      @interf.register_prec( :Nonassoc, val[1] )
   result
  end
 }

 module_eval( '', '(build)', 143 ) {
  def _reduce_16( val, _values, result )
      result = val
   result
  end
 }

 module_eval( '', '(build)', 147 ) {
  def _reduce_17( val, _values, result )
      result.push val[1]
   result
  end
 }

 module_eval( '', '(build)', 152 ) {
  def _reduce_18( val, _values, result )
      result = @interf.get_token( val[0] )
   result
  end
 }

 module_eval( '', '(build)', 156 ) {
  def _reduce_19( val, _values, result )
      result = @interf.get_token( eval '"' + val[0] + '"' )
   result
  end
 }

 # reduce 20 omitted

 # reduce 21 omitted

 module_eval( '', '(build)', 166 ) {
  def _reduce_22( val, _values, result )
      @interf.register_rule @interf.get_token(val[0]), val[2]
   result
  end
 }

 module_eval( '', '(build)', 170 ) {
  def _reduce_23( val, _values, result )
      @interf.register_rule nil, val[2]
   result
  end
 }

 module_eval( '', '(build)', 175 ) {
  def _reduce_24( val, _values, result )
      result = []
   result
  end
 }

 module_eval( '', '(build)', 179 ) {
  def _reduce_25( val, _values, result )
      result.push val[1] if val[1]
   result
  end
 }

 module_eval( '', '(build)', 184 ) {
  def _reduce_26( val, _values, result )
      result = @interf.get_token( val[0] )
   result
  end
 }

 module_eval( '', '(build)', 188 ) {
  def _reduce_27( val, _values, result )
      result = @interf.get_token( eval %<"#{result}"> )
   result
  end
 }

 module_eval( '', '(build)', 192 ) {
  def _reduce_28( val, _values, result )
      result = Action.new( *result )
   result
  end
 }

 module_eval( '', '(build)', 196 ) {
  def _reduce_29( val, _values, result )
      @interf.register_tmpprec val[1]
      result = nil
   result
  end
 }

  end   # class RaccParser

end   # module Racc
